package PureMessage::HTTPD::Services::Logsearch;
use strict;
use warnings;
use base qw( PureMessage::HTTPD::Services::Application );
use PureMessage::HTTPD::Services::Access qw(ip_is_valid);
use Apache::Constants qw(FORBIDDEN);
use Sophos::LogSearch::Local;
use Data::Dumper;

# Setup; specify name of logfile to use and list of available run modes.
sub setup {
    my $self = shift;
    $self->start_mode('message');
    $self->run_modes(
        # LogSearch runmodes
        query     => 'show_logsearch_results',
        message   => 'show_logsearch_message',
        denied    => 'show_error',
    );

    $self->header_props(-type => 'text/plain');
}

sub cgiapp_prerun {
    my $self = shift;

    my $r = $self->param('r') || die "Can't find apache request object!";
    my $c = $r->connection;
    my $ip = $c->remote_ip;
    unless (ip_is_valid($ip)) {
        warn "Access by unauthorized ip attempted: $ip\n";
        $r->status_line("403 Your ip is not in the authorized ip list.  Access from '$ip' is denied.");
        $self->prerun_mode('denied');
        return;
    }
}

sub show_error {
    return;
}

sub show_logsearch_message {
    my ($self) = @_;

    my $s = Sophos::LogSearch::Local->new();
    return '' unless defined($s);
    my %args = $self->query->Vars();

    return '' unless $args{key};
    
    my $message = $s->message($args{key});
    
    return '' unless $message;
    my $raw = $message->rawlog();
    my $msgl = $message->message_log_data()
        unless $args{no_message_log};

    my $return_data = $message->{mline};
    $return_data .= "\n-";
    $return_data .= "\n$raw" if $raw;
    $return_data .= "\n-";
    $return_data .= "\n$msgl" if $msgl;

    return $return_data;
}

sub show_logsearch_results {
    my ($self) = @_;

    my $s = Sophos::LogSearch::Local->new();
    return '' unless defined($s);
    my %args = $self->query->Vars();

    # Defaults:
    $args{count}  ||= 1000;
    $args{offset} ||= 0;
    $args{sort}   ||= '-time';

    my $search_results = $s->search(%args);
    return '' unless scalar(@{$search_results});

    my $results = [ map { 
        my $fx = $_->{mline}; 
        chomp $fx; 
        $fx; 
      } @{$search_results} ];
    return join("\n", @{$results});
}

1;
